@@ -1,11 +1,6 @@
-try:
-    from importlib.metadata import version
-except ModuleNotFoundError:
-    from importlib_metadata import version
-
 import json
 import re
-
+from ploomber_core.exceptions import modify_exceptions
 from IPython.core.magic import (
     Magics,
     cell_magic,
@@ -20,6 +15,9 @@ import sql.connection
 import sql.parse
 import sql.run
 from sql.store import store
+from sql.command import SQLCommand
+from sql.magic_plot import SqlPlotMagic
+from sql.magic_cmd import SqlCmdMagic
 
 try:
     from traitlets.config.configurable import Configurable
@@ -33,17 +31,15 @@ except ImportError:
     DataFrame = None
     Series = None
 
-from ploomber_core.telemetry.telemetry import Telemetry
-
-telemetry = Telemetry(
-    api_key="phc_P9SpSeypyPwxrMdFn2edOOEooQioF2axppyEeDwtMSP",
-    package_name="jupysql",
-    version=version("jupysql"),
-)
+from sql.telemetry import telemetry
 
 
 @magics_class
 class RenderMagic(Magics):
+    """
+    %sqlrender magic which prints composed queries
+    """
+
     @line_magic
     @magic_arguments()
     # TODO: only accept one arg
@@ -56,6 +52,7 @@ class RenderMagic(Magics):
         action="append",
         dest="with_",
     )
+    @telemetry.log_call("sqlrender")
     def sqlrender(self, line):
         args = parse_argstring(self.sqlrender, line)
         return str(store[args.line[0]])
@@ -67,7 +64,7 @@ class SqlMagic(Magics, Configurable):
 
     Provides the %%sql magic."""
 
-    displaycon = Bool(True, config=True, help="Show connection string after execute")
+    displaycon = Bool(True, config=True, help="Show connection string after execution")
     autolimit = Int(
         0,
         config=True,
@@ -77,7 +74,10 @@ class SqlMagic(Magics, Configurable):
     style = Unicode(
         "DEFAULT",
         config=True,
-        help="Set the table printing style to any of prettytable's defined styles (currently DEFAULT, MSWORD_FRIENDLY, PLAIN_COLUMNS, RANDOM)",
+        help=(
+            "Set the table printing style to any of prettytable's "
+            "defined styles (currently DEFAULT, MSWORD_FRIENDLY, PLAIN_COLUMNS, RANDOM)"
+        ),
     )
     short_errors = Bool(
         True,
@@ -88,7 +88,10 @@ class SqlMagic(Magics, Configurable):
         None,
         config=True,
         allow_none=True,
-        help="Automatically limit the number of rows displayed (full result set is still stored)",
+        help=(
+            "Automatically limit the number of rows "
+            "displayed (full result set is still stored)"
+        ),
     )
     autopandas = Bool(
         False,
@@ -109,8 +112,8 @@ class SqlMagic(Magics, Configurable):
     )
     autocommit = Bool(True, config=True, help="Set autocommit mode")
 
+    @telemetry.log_call("init")
     def __init__(self, shell):
-        telemetry.log_api("sql-magic-init")
 
         self._store = store
 
@@ -123,6 +126,8 @@ class SqlMagic(Magics, Configurable):
     @needs_local_scope
     @line_magic("sql")
     @cell_magic("sql")
+    @line_magic("jupysql")
+    @cell_magic("jupysql")
     @magic_arguments()
     @argument("line", default="", nargs="*", type=str, help="sql")
     @argument(
@@ -180,8 +185,17 @@ class SqlMagic(Magics, Configurable):
         action="store_true",
         help="Do not execute query (use it with --save)",
     )
+    @argument(
+        "-A",
+        "--alias",
+        type=str,
+        help="Assign an alias to the connection",
+    )
+    @telemetry.log_call("execute")
     def execute(self, line="", cell="", local_ns={}):
-        """Runs SQL statement against a database, specified by SQLAlchemy connect string.
+        """
+        Runs SQL statement against a database, specified by
+        SQLAlchemy connect string.
 
         If no database connection has been established, first word
         should be a SQLAlchemy connection string, or the user@db name
@@ -205,38 +219,33 @@ class SqlMagic(Magics, Configurable):
           mysql+pymysql://me:mypw@localhost/mydb
 
         """
+        # line is the text after the magic, cell is the cell's body
 
-        # Parse variables (words wrapped in {}) for %%sql magic (for %sql this is done automatically)
-        cell = self.shell.var_expand(cell)
-        line = sql.parse.without_sql_comment(parser=self.execute.parser, line=line)
-        args = parse_argstring(self.execute, line)
+        # Examples
+
+        # %sql {line}
+        # note that line magic has no body
+
+        # %%sql {line}
+        # {cell}
+
+        # save globals and locals so they can be referenced in bind vars
+        user_ns = self.shell.user_ns.copy()
+        user_ns.update(local_ns)
+
+        command = SQLCommand(self, user_ns, line, cell)
+        # args.line: contains the line after the magic with all options removed
+        args = command.args
 
         if args.connections:
             return sql.connection.Connection.connections
         elif args.close:
             return sql.connection.Connection._close(args.close)
 
-        # save globals and locals so they can be referenced in bind vars
-        user_ns = self.shell.user_ns.copy()
-        user_ns.update(local_ns)
+        connect_arg = command.connection
 
-        command_text = " ".join(args.line) + "\n" + cell
-
-        if args.file:
-            with open(args.file, "r") as infile:
-                command_text = infile.read() + "\n" + command_text
-
-        parsed = sql.parse.parse(command_text, self)
-
-        original = parsed["sql"]
-
-        if args.with_:
-            final = self._store.render(original, with_=args.with_)
-            parsed["sql"] = str(final)
-
-        connect_str = parsed["connection"]
         if args.section:
-            connect_str = sql.parse.connection_from_dsn_section(args.section, self)
+            connect_arg = sql.parse.connection_from_dsn_section(args.section, self)
 
         if args.connection_arguments:
             try:
@@ -258,11 +267,14 @@ class SqlMagic(Magics, Configurable):
             args.creator = user_ns[args.creator]
 
         try:
+            # this creates a new connection or use an existing one
+            # depending on the connect_arg value
             conn = sql.connection.Connection.set(
-                connect_str,
+                connect_arg,
                 displaycon=self.displaycon,
                 connect_args=args.connection_arguments,
                 creator=args.creator,
+                alias=args.alias,
             )
         except Exception as e:
             print(e)
@@ -271,27 +283,27 @@ class SqlMagic(Magics, Configurable):
 
         if args.persist:
             return self._persist_dataframe(
-                parsed["sql"], conn, user_ns, append=False, index=not args.no_index
+                command.sql, conn, user_ns, append=False, index=not args.no_index
             )
 
         if args.append:
             return self._persist_dataframe(
-                parsed["sql"], conn, user_ns, append=True, index=not args.no_index
+                command.sql, conn, user_ns, append=True, index=not args.no_index
             )
 
-        if not parsed["sql"]:
+        if not command.sql:
             return
 
         # store the query if needed
         if args.save:
-            self._store.store(args.save, original, with_=args.with_)
+            self._store.store(args.save, command.sql_original, with_=args.with_)
 
         if args.no_execute:
             print("Skipping execution...")
             return
 
         try:
-            result = sql.run.run(conn, parsed["sql"], self, user_ns)
+            result = sql.run.run(conn, command.sql, self, user_ns)
 
             if (
                 result is not None
@@ -317,10 +329,8 @@ class SqlMagic(Magics, Configurable):
                 return None
             else:
 
-                if parsed["result_var"]:
-                    result_var = parsed["result_var"]
-                    print("Returning data to local variable {}".format(result_var))
-                    self.shell.user_ns.update({result_var: result})
+                if command.result_var:
+                    self.shell.user_ns.update({command.result_var: result})
                     return None
 
                 # Return results into the default ipython _ variable
@@ -329,6 +339,7 @@ class SqlMagic(Magics, Configurable):
         # JA: added DatabaseError for MySQL
         except (ProgrammingError, OperationalError, DatabaseError) as e:
             # Sqlite apparently return all errors as OperationalError :/
+
             if self.short_errors:
                 print(e)
             else:
@@ -336,6 +347,7 @@ class SqlMagic(Magics, Configurable):
 
     legal_sql_identifier = re.compile(r"^[A-Za-z0-9#_$]+")
 
+    @modify_exceptions
     def _persist_dataframe(self, raw, conn, user_ns, append=False, index=True):
         """Implements PERSIST, which writes a DataFrame to the RDBMS"""
         if not DataFrame:
@@ -369,7 +381,9 @@ def load_ipython_extension(ip):
     # this fails in both Firefox and Chrome for OS X.
     # I get the error: TypeError: IPython.CodeCell.config_defaults is undefined
 
-    # js = "IPython.CodeCell.config_defaults.highlight_modes['magic_sql'] = {'reg':[/^%%sql/]};"
+    # js = "IPython.CodeCell.config_defaults.highlight_modes['magic_sql'] = {'reg':[/^%%sql/]};" # noqa
     # display_javascript(js, raw=True)
     ip.register_magics(SqlMagic)
     ip.register_magics(RenderMagic)
+    ip.register_magics(SqlPlotMagic)
+    ip.register_magics(SqlCmdMagic)
